# This file is generated by objective.metadata
#
# Last update: Sun Feb 20 18:58:06 2022
#
# flake8: noqa

import objc, sys
from typing import NewType

if sys.maxsize > 2**32:

    def sel32or64(a, b):
        return b

else:

    def sel32or64(a, b):
        return a


if objc.arch == "arm64":

    def selAorI(a, b):
        return a

else:

    def selAorI(a, b):
        return b


misc = {}
constants = """$LAErrorDomain$LATouchIDAuthenticationMaximumAllowableReuseDuration@d$"""
enums = """$LAAccessControlOperationCreateItem@0$LAAccessControlOperationCreateKey@2$LAAccessControlOperationUseItem@1$LAAccessControlOperationUseKeyDecrypt@4$LAAccessControlOperationUseKeyKeyExchange@5$LAAccessControlOperationUseKeySign@3$LABiometryNone@0$LABiometryTypeFaceID@2$LABiometryTypeNone@0$LABiometryTypeTouchID@1$LACredentialTypeApplicationPassword@0$LACredentialTypeSmartCardPIN@-3$LAErrorAppCancel@-9$LAErrorAuthenticationFailed@-1$LAErrorBiometryDisconnected@-13$LAErrorBiometryLockout@-8$LAErrorBiometryNotAvailable@-6$LAErrorBiometryNotEnrolled@-7$LAErrorBiometryNotPaired@-12$LAErrorInvalidContext@-10$LAErrorInvalidDimensions@-14$LAErrorNotInteractive@-1004$LAErrorPasscodeNotSet@-5$LAErrorSystemCancel@-4$LAErrorTouchIDLockout@-8$LAErrorTouchIDNotAvailable@-6$LAErrorTouchIDNotEnrolled@-7$LAErrorUserCancel@-2$LAErrorUserFallback@-3$LAErrorWatchNotAvailable@-11$LAPolicyDeviceOwnerAuthentication@2$LAPolicyDeviceOwnerAuthenticationWithBiometrics@1$LAPolicyDeviceOwnerAuthenticationWithBiometricsOrWatch@4$LAPolicyDeviceOwnerAuthenticationWithWatch@3$kLACredentialCTKPIN@-3$kLACredentialSmartCardPIN@-3$kLACredentialTypeApplicationPassword@0$kLACredentialTypePasscode@-1$kLACredentialTypePassphrase@-2$kLAErrorAppCancel@-9$kLAErrorAuthenticationFailed@-1$kLAErrorBiometryDisconnected@-13$kLAErrorBiometryNotPaired@-12$kLAErrorInvalidContext@-10$kLAErrorInvalidDimensions@-14$kLAErrorNotInteractive@-1004$kLAErrorPasscodeNotSet@-5$kLAErrorSystemCancel@-4$kLAErrorTouchIDLockout@-8$kLAErrorTouchIDNotAvailable@-6$kLAErrorTouchIDNotEnrolled@-7$kLAErrorUserCancel@-2$kLAErrorUserFallback@-3$kLAErrorWatchNotAvailable@-11$kLAOptionAuthenticationReason@2$kLAOptionUserFallback@1$kLAPolicyDeviceOwnerAuthentication@2$kLAPolicyDeviceOwnerAuthenticationWithBiometrics@1$kLAPolicyDeviceOwnerAuthenticationWithBiometricsOrWatch@4$kLAPolicyDeviceOwnerAuthenticationWithWatch@3$"""
misc.update(
    {
        "LABiometryType": NewType("LABiometryType", int),
        "LAAccessControlOperation": NewType("LAAccessControlOperation", int),
        "LACredentialType": NewType("LACredentialType", int),
        "LAPolicy": NewType("LAPolicy", int),
        "LAError": NewType("LAError", int),
    }
)
misc.update({"kLAErrorDomain": "com.apple.LocalAuthentication"})
aliases = {
    "LACredentialTypeSmartCardPIN": "kLACredentialSmartCardPIN",
    "LAErrorBiometryLockout": "kLAErrorBiometryLockout",
    "LAErrorBiometryNotEnrolled": "kLAErrorBiometryNotEnrolled",
    "LABiometryNone": "LABiometryTypeNone",
    "kLAErrorBiometryNotEnrolled": "kLAErrorTouchIDNotEnrolled",
    "kLAErrorBiometryNotAvailable": "kLAErrorTouchIDNotAvailable",
    "LAErrorBiometryNotAvailable": "kLAErrorBiometryNotAvailable",
    "kLAErrorBiometryLockout": "kLAErrorTouchIDLockout",
}
r = objc.registerMetaDataForSelector
objc._updatingMetadata(True)
try:
    r(
        b"LAContext",
        b"canEvaluatePolicy:error:",
        {"retval": {"type": b"Z"}, "arguments": {3: {"type_modifier": b"o"}}},
    )
    r(
        b"LAContext",
        b"evaluateAccessControl:operation:localizedReason:reply:",
        {
            "arguments": {
                5: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"Z"},
                            2: {"type": b"@"},
                        },
                    }
                }
            }
        },
    )
    r(
        b"LAContext",
        b"evaluatePolicy:localizedReason:reply:",
        {
            "arguments": {
                4: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"Z"},
                            2: {"type": b"@"},
                        },
                    }
                }
            }
        },
    )
    r(
        b"LAContext",
        b"evaluatePolicy:operation:localizedReason:reply:",
        {
            "arguments": {
                5: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"Z"},
                            2: {"type": b"@"},
                        },
                    }
                }
            }
        },
    )
    r(b"LAContext", b"interactionNotAllowed", {"retval": {"type": "Z"}})
    r(b"LAContext", b"isCancelButtonVisible", {"retval": {"type": "Z"}})
    r(b"LAContext", b"isCredentialSet:", {"retval": {"type": "Z"}})
    r(b"LAContext", b"isFallbackButtonVisible", {"retval": {"type": "Z"}})
    r(b"LAContext", b"setCancelButtonVisible:", {"arguments": {2: {"type": "Z"}}})
    r(b"LAContext", b"setCredential:type:", {"retval": {"type": "Z"}})
    r(b"LAContext", b"setFallbackButtonVisible:", {"arguments": {2: {"type": "Z"}}})
    r(b"LAContext", b"setInteractionNotAllowed:", {"arguments": {2: {"type": "Z"}}})
finally:
    objc._updatingMetadata(False)
expressions = {}

# END OF FILE



from PyObjCTools.TestSupport import TestCase, min_os_level
import LocalAuthentication


class TestLAContext(TestCase):
    def test_enum_types(self):
        self.assertIsEnumType(LocalAuthentication.LAError)

    @min_os_level("10.10")
    def testConstants(self):
        self.assertEqual(
            LocalAuthentication.LAErrorAuthenticationFailed,
            LocalAuthentication.kLAErrorAuthenticationFailed,
        )
        self.assertEqual(
            LocalAuthentication.LAErrorUserCancel,
            LocalAuthentication.kLAErrorUserCancel,
        )
        self.assertEqual(
            LocalAuthentication.LAErrorUserFallback,
            LocalAuthentication.kLAErrorUserFallback,
        )
        self.assertEqual(
            LocalAuthentication.LAErrorSystemCancel,
            LocalAuthentication.kLAErrorSystemCancel,
        )
        self.assertEqual(
            LocalAuthentication.LAErrorPasscodeNotSet,
            LocalAuthentication.kLAErrorPasscodeNotSet,
        )
        self.assertEqual(
            LocalAuthentication.LAErrorTouchIDNotAvailable,
            LocalAuthentication.kLAErrorTouchIDNotAvailable,
        )
        self.assertEqual(
            LocalAuthentication.LAErrorTouchIDNotEnrolled,
            LocalAuthentication.kLAErrorTouchIDNotEnrolled,
        )
        self.assertEqual(
            LocalAuthentication.LAErrorTouchIDLockout,
            LocalAuthentication.kLAErrorTouchIDLockout,
        )
        self.assertEqual(
            LocalAuthentication.LAErrorAppCancel, LocalAuthentication.kLAErrorAppCancel
        )
        self.assertEqual(
            LocalAuthentication.LAErrorInvalidContext,
            LocalAuthentication.kLAErrorInvalidContext,
        )
        self.assertEqual(
            LocalAuthentication.LAErrorNotInteractive,
            LocalAuthentication.kLAErrorNotInteractive,
        )

        self.assertEqual(
            LocalAuthentication.LAErrorBiometryNotAvailable,
            LocalAuthentication.kLAErrorBiometryNotAvailable,
        )
        self.assertEqual(
            LocalAuthentication.LAErrorBiometryNotEnrolled,
            LocalAuthentication.kLAErrorBiometryNotEnrolled,
        )
        self.assertEqual(
            LocalAuthentication.LAErrorBiometryLockout,
            LocalAuthentication.kLAErrorBiometryLockout,
        )

        self.assertEqual(
            LocalAuthentication.LAErrorWatchNotAvailable,
            LocalAuthentication.kLAErrorWatchNotAvailable,
        )
        self.assertEqual(
            LocalAuthentication.LAErrorBiometryNotPaired,
            LocalAuthentication.kLAErrorBiometryNotPaired,
        )
        self.assertEqual(
            LocalAuthentication.LAErrorBiometryDisconnected,
            LocalAuthentication.kLAErrorBiometryDisconnected,
        )
        self.assertEqual(
            LocalAuthentication.LAErrorInvalidDimensions,
            LocalAuthentication.kLAErrorInvalidDimensions,
        )

    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertIsInstance(LocalAuthentication.LAErrorDomain, str)



import objc

from PyObjCTools.TestSupport import TestCase, min_os_level, os_level_between
import LocalAuthentication
import Security
import Foundation


class TestLAContext(TestCase):
    def test_enum_types(self):
        self.assertIsEnumType(LocalAuthentication.LAAccessControlOperation)
        self.assertIsEnumType(LocalAuthentication.LABiometryType)
        self.assertIsEnumType(LocalAuthentication.LACredentialType)
        self.assertIsEnumType(LocalAuthentication.LAPolicy)

    @min_os_level("10.10")
    def testClasses(self):
        self.assertIsInstance(LocalAuthentication.LAContext, objc.objc_class)

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(LocalAuthentication.LAContext.canEvaluatePolicy_error_)
        self.assertArgIsOut(LocalAuthentication.LAContext.canEvaluatePolicy_error_, 1)

        self.assertArgIsBlock(
            LocalAuthentication.LAContext.evaluatePolicy_localizedReason_reply_,
            2,
            b"v" + objc._C_NSBOOL + b"@",
        )

    @os_level_between("10.10", "10.15")
    def testMethods10_10_to_15(self):
        self.assertResultIsBOOL(LocalAuthentication.LAContext.isCancelButtonVisible)
        self.assertArgIsBOOL(LocalAuthentication.LAContext.setCancelButtonVisible_, 0)

        self.assertResultIsBOOL(LocalAuthentication.LAContext.isFallbackButtonVisible)
        self.assertArgIsBOOL(LocalAuthentication.LAContext.setFallbackButtonVisible_, 0)

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertArgIsBOOL(LocalAuthentication.LAContext.setInteractionNotAllowed_, 0)
        self.assertResultIsBOOL(LocalAuthentication.LAContext.interactionNotAllowed)

        # self.assertArgIsBlock(LocalAuthentication.LAContext.withCurrentContextExecute_queue_, 0, b"v")

    def testConstants(self):
        self.assertEqual(LocalAuthentication.LABiometryTypeNone, 0)
        self.assertEqual(LocalAuthentication.LABiometryNone, 0)
        self.assertEqual(LocalAuthentication.LABiometryTypeTouchID, 1)
        self.assertEqual(LocalAuthentication.LABiometryTypeFaceID, 2)

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertEqual(
            LocalAuthentication.LAPolicyDeviceOwnerAuthenticationWithBiometrics,
            LocalAuthentication.kLAPolicyDeviceOwnerAuthenticationWithBiometrics,
        )
        self.assertEqual(
            LocalAuthentication.LAPolicyDeviceOwnerAuthentication,
            LocalAuthentication.kLAPolicyDeviceOwnerAuthentication,
        )
        self.assertEqual(
            LocalAuthentication.LACredentialTypeApplicationPassword,
            LocalAuthentication.kLACredentialTypeApplicationPassword,
        )

    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertEqual(LocalAuthentication.LAAccessControlOperationCreateItem, 0)
        self.assertEqual(LocalAuthentication.LAAccessControlOperationUseItem, 1)
        self.assertEqual(LocalAuthentication.LAAccessControlOperationCreateKey, 2)
        self.assertEqual(LocalAuthentication.LAAccessControlOperationUseKeySign, 3)

    @min_os_level("10.15")
    def test_constants10_15(self):
        self.assertEqual(
            LocalAuthentication.LACredentialTypeSmartCardPIN,
            LocalAuthentication.kLACredentialSmartCardPIN,
        )

    @min_os_level("10.12")
    def testConstants10_12(self):
        self.assertEqual(LocalAuthentication.LAAccessControlOperationUseKeyDecrypt, 4)
        self.assertEqual(
            LocalAuthentication.LAAccessControlOperationUseKeyKeyExchange, 5
        )

        self.assertIsInstance(
            LocalAuthentication.LATouchIDAuthenticationMaximumAllowableReuseDuration,
            float,
        )

    @min_os_level("10.15")
    def testConstants10_15(self):
        self.assertEqual(
            LocalAuthentication.LAPolicyDeviceOwnerAuthenticationWithWatch,
            LocalAuthentication.kLAPolicyDeviceOwnerAuthenticationWithWatch,
        )
        self.assertEqual(
            LocalAuthentication.LAPolicyDeviceOwnerAuthenticationWithBiometricsOrWatch,
            LocalAuthentication.kLAPolicyDeviceOwnerAuthenticationWithBiometricsOrWatch,
        )

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertResultIsBOOL(LocalAuthentication.LAContext.setCredential_type_)
        self.assertResultIsBOOL(LocalAuthentication.LAContext.isCredentialSet_)

        self.assertArgIsBlock(
            LocalAuthentication.LAContext.evaluatePolicy_localizedReason_reply_,
            2,
            b"v" + objc._C_NSBOOL + b"@",
        )

    def test_regr_security_types(self):
        # Issue #324
        auth_ctx = LocalAuthentication.LAContext.new()
        access_control = Security.SecAccessControlCreateWithFlags(
            None,
            Security.kSecAttrAccessibleWhenUnlocked,
            Security.kSecAccessControlUserPresence,
            None,
        )[0]

        called = False

        def callback(a, b):
            nonlocal called
            called = True

        auth_ctx.evaluateAccessControl_operation_localizedReason_reply_(
            access_control,
            LocalAuthentication.LAAccessControlOperationCreateKey,
            "test",
            callback,
        )

        loop = Foundation.NSRunLoop.currentRunLoop()
        loop.runUntilDate_(Foundation.NSDate.dateWithTimeIntervalSinceNow_(0.2))

        self.assertTrue(called)



from PyObjCTools.TestSupport import TestCase, min_os_level
import LocalAuthentication


class TestLAPublicDefines(TestCase):
    @min_os_level("10.10")
    def testConstants(self):
        self.assertEqual(LocalAuthentication.kLACredentialTypeApplicationPassword, 0)
        self.assertEqual(LocalAuthentication.kLAErrorNotInteractive, -1004)

        self.assertEqual(
            LocalAuthentication.kLAPolicyDeviceOwnerAuthenticationWithBiometrics, 1
        )
        self.assertEqual(LocalAuthentication.kLAPolicyDeviceOwnerAuthentication, 2)
        self.assertEqual(LocalAuthentication.kLAOptionUserFallback, 1)
        self.assertEqual(LocalAuthentication.kLAOptionAuthenticationReason, 2)
        self.assertEqual(
            LocalAuthentication.kLAPolicyDeviceOwnerAuthenticationWithWatch, 3
        )
        self.assertEqual(
            LocalAuthentication.kLAPolicyDeviceOwnerAuthenticationWithBiometricsOrWatch,
            4,
        )

        self.assertEqual(LocalAuthentication.kLACredentialTypePasscode, -1)
        self.assertEqual(LocalAuthentication.kLACredentialTypePassphrase, -2)
        self.assertEqual(LocalAuthentication.kLACredentialCTKPIN, -3)
        self.assertEqual(LocalAuthentication.kLACredentialSmartCardPIN, -3)

        self.assertEqual(LocalAuthentication.kLAErrorAuthenticationFailed, -1)
        self.assertEqual(LocalAuthentication.kLAErrorUserCancel, -2)
        self.assertEqual(LocalAuthentication.kLAErrorUserFallback, -3)
        self.assertEqual(LocalAuthentication.kLAErrorSystemCancel, -4)
        self.assertEqual(LocalAuthentication.kLAErrorPasscodeNotSet, -5)
        self.assertEqual(LocalAuthentication.kLAErrorTouchIDNotAvailable, -6)
        self.assertEqual(LocalAuthentication.kLAErrorTouchIDNotEnrolled, -7)
        self.assertEqual(LocalAuthentication.kLAErrorTouchIDLockout, -8)
        self.assertEqual(LocalAuthentication.kLAErrorAppCancel, -9)
        self.assertEqual(LocalAuthentication.kLAErrorInvalidContext, -10)
        self.assertEqual(LocalAuthentication.kLAErrorWatchNotAvailable, -11)
        self.assertEqual(LocalAuthentication.kLAErrorBiometryNotPaired, -12)
        self.assertEqual(LocalAuthentication.kLAErrorBiometryDisconnected, -13)
        self.assertEqual(LocalAuthentication.kLAErrorInvalidDimensions, -14)
        self.assertEqual(
            LocalAuthentication.kLAErrorDomain, "com.apple.LocalAuthentication"
        )

        self.assertEqual(
            LocalAuthentication.kLAErrorBiometryNotAvailable,
            LocalAuthentication.kLAErrorTouchIDNotAvailable,
        )
        self.assertEqual(
            LocalAuthentication.kLAErrorBiometryNotEnrolled,
            LocalAuthentication.kLAErrorTouchIDNotEnrolled,
        )
        self.assertEqual(
            LocalAuthentication.kLAErrorBiometryLockout,
            LocalAuthentication.kLAErrorTouchIDLockout,
        )
